;; Copyright 2007, Greg Detre, Per Sederberg.


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; test version of freex-conf. run this with 'emacs -q'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; if you're starting a new testdocs, then in ipython:
;;   from freex_sqlalchemy import * ; fsqa = create_fsqa('/home/greg/elisp/freex/testdocs/froox.db',False,False);


(add-to-list 'load-path "/home/greg/elisp/freex")

(require 'freex-hiert)

(autoload 'pymacs-load "pymacs" nil t)
(autoload 'pymacs-eval "pymacs" nil t)
(autoload 'pymacs-apply "pymacs")
(autoload 'pymacs-call "pymacs")
(eval-after-load "pymacs"
  '(add-to-list 'pymacs-load-path "/home/greg/elisp/freex/"))

;; where the freex .db file and its .freex friends will live
(setq freex-mode-dir "/home/greg/elisp/freex/testdocs/")

;; this needs to be set in the freex_sqlalchemy file_ext
;; class variable. the default is "freex". no dot needed
;; (freex-sqlalchemy-put-ext "froox")
(setq freex-mode-ext "fraax")

;; this determines whether things are stored in the db
;;
;; unfortunately, mirroring files to db doesn't appear to be
;; working correctly at the moment
;;
(setq freex-content-storage "mirror-files-to-db")
;; (setq freex-content-storage "files")

(load "/home/greg/elisp/freex/freex-mode.el")

;; xxx shouldn't need this on top of the defvar freex-mode-ext
;; (freex-sqlalchemy-put-ext "froox")
;; (freex-sqlalchemy-get-ext)
;; (freex-sqlalchemy-get-all-filenames)

;; i have C-l for find-file and M-l for freex-find-file
(global-set-key [(meta l)] 'freex-find-file)

;; for tagging
;;(global-set-key (quote [C-M-return])
;;                 'freex-meta-edit-tag-parents-in-minibuffer)

(add-hook 'freex-mode-hook (lambda () (auto-fill-mode 1)))

;; set it to load freex-mode when it sees a .freex
;; file. shouldn't this be in freex.el somewhere???
(setq auto-mode-alist
      (cons '("\\.fraax\\'" . freex-mode)
            auto-mode-alist))

;; keybindings for hierarchical tabbing
(define-key freex-mode-map (kbd "RET") 'freex-hiert-newline-and-indent)
(define-key freex-mode-map [(shift return)] 'newline)

(define-key freex-mode-map [(meta return)]
  'freex-meta-define-new-or-insert-metadata)


;; (global-unset-key (quote [201326641])) ;; control meta 1
(define-key freex-mode-map
  (quote [201326641]) ;; [(control meta 1)]
  'freex-fontify-insert-sect-element-1)
(define-key freex-mode-map
  (quote [201326642]) ;; [(control meta 2)]
  'freex-fontify-insert-sect-element-2)
(define-key freex-mode-map
  (quote [201326643]) ;; [(control meta 3)]
  'freex-fontify-insert-sect-element-3)


;; for find-alias etc.
(setq completion-ignore-case t)

;; make it a little more visible than the default 20
(setq freex-embed-color-step 25)

;; get rid of the metadata overlays when we save
;; (add-hook 'freex-embed-save-hook
;;           'freex-meta-remove-meta-overlays)

;; this automatically tokenizes the filename by hyphens, and
;; adds all the parts as tags
(add-hook 'freex-meta-add-nugget-hooks
          'freex-meta-parse-alias-into-tag-parents)

;; ;; when i imported all my old notes, i wanted them to be
;; ;; tagged as such
;; (add-hook 'freex-meta-add-nugget-hooks
;;           'freex-meta-add-specific-tag)
;; (setq freex-conf-specific-tag-to-add "old notes")


(defvar freex-conf-specific-tag-to-add nil
"If you add freex-conf-add-specific-tag as a hook to
freex-meta-add-nugget-hooks, then it will add this to all new
nuggets.")


(defun freex-meta-add-specific-tag (alias)
  "Add the freex-conf-specific-tag-to-add alias to the list of
ALIAS's tag-parents. If you add this function as a hook to the
freex-meta-add-nugget-hooks list, then it's a good way of
batch-tagging a bunch of nuggets you want to create at once."
  (freex-sqlalchemy-add-tag-child-a-to-tag-parent-a
   alias
   freex-conf-specific-tag-to-add))


(defun freex-todo ()
  (interactive)
  "Automatically load my elisp/freex/todo.muse file."
  (find-file "/home/greg/elisp/freex/todo.muse"))
;; (global-set-key 


(defun freex-meta-count-untagged ()
  "This is so that when you're batch-tagging by hand, you can see
how much work you have left to do."
  (interactive)
  (message (format "%i untagged"
                   (freex-sqlalchemy-count-untagged))))


(defun freex-meta-find-random-untagged ()
  "If you want to tidy up your database by looking for untagged
nuggets - will choose an untagged nugget and open it for you. I
just call this over and over until I'm bored."
  (interactive)
  (find-file
   (concat (file-name-as-directory freex-mode-dir)
           (freex-sqlalchemy-choose-random-untagged)))
  (freex-meta-insert-metadata-here))


;; this is the regex for finding files in the directory that
;; should be inserted into the db. the caret at the
;; beginning tells it to ignore files that have a dot in
;; front (useful for excluding emacs temp files), but it
;; also excludes any files with dots in them after the first
;; character, which annoys per. since my filenames don't
;; have dots in them, this is fine for me
(setq freex-mode-dir-filter "^[^.#]+\\.fraax$")


;; gave me a NeedsExtension error
;; (add-hook 'muse-mode-hook (lambda () (freex-mode)))


(defun freex-clean-up-punctuation ()
  "Pymacs dies if you try and save a buffer with weird
punctuation. This will replace all of that with standard
characters, and then restart freex."
  (interactive)
  (replace-string "…" "..." nil (point-min) (point-max))
  (replace-string "”" "\"" nil (point-min) (point-max))
  (replace-string "“" "\"" nil (point-min) (point-max))
  (replace-string "’" "'" nil (point-min) (point-max))
  (replace-string "–" "-" nil (point-min) (point-max))
  (replace-string " " "_" nil (point-min) (point-max))
  (replace-string "‘" "'" nil (point-min) (point-max))
  (replace-string "¯" "-" nil (point-min) (point-max))
  (replace-string "­" "-" nil (point-min) (point-max))
  (replace-string "·" "*" nil (point-min) (point-max))
  (replace-string "”" "\"" nil (point-min) (point-max))
  (replace-string "“" "\"" nil (point-min) (point-max))
  (replace-string "‘" "'" nil (point-min) (point-max))
  (replace-string "’" "'" nil (point-min) (point-max))
  (replace-string "½" ".5" nil (point-min) (point-max))
  (replace-string "\x92¡Ç" "'" nil (point-min) (point-max))
  (replace-string "…" "..." nil (point-min) (point-max))
  (replace-string "®" "->" nil (point-min) (point-max))
  ;; (freex-restart)
  ;; (freex-restart)
  ;; i don't know why we need to restart it more than once,
  ;; but the first time often doesn't quite seem enough
  ;; (freex-restart)
  ;; this should trigger freex-embed-save, if you're in freex mode
  ;; (save-buffer)
  (freex-restart)
  )




